@@ -292,7 +292,7 @@ enum aws_s3_checksum_location {
     AWS_SCL_TRAILER,
 };
 
-enum aws_s3_recv_file_option {
+enum aws_s3_recv_file_options {
     /**
      * Create a new file if it doesn't exist, otherwise replace the existing file.
      */
@@ -314,6 +314,43 @@ enum aws_s3_recv_file_option {
      */
     AWS_S3_RECV_FILE_WRITE_TO_POSITION,
 };
+
+/**
+ * WARNING: experimental/unstable:
+ * Controls how client performance file I/O operations. Only applies to the file based workload.
+ **/
+struct aws_s3_file_io_options {
+    /**
+     * Skip buffering the part in memory before sending the request.
+     *
+     * Default to false on small objects, and true when the object size exceed a certain threshold
+     *`g_streaming_object_size_threshold`.
+     **/
+    bool should_stream;
+
+    /**
+     * The estimated disk throughput. Only be applied when `streaming_upload` is true.
+     * in gigabits per second (Gbps).
+     *
+     * Notes: There are possibilities that cannot reach the all available disk throughput:
+     * 1. Disk is busy with other applications
+     * 2. OS Cache may cap the throughput, use `direct_io` to get around this.
+     *
+     * Default to throughput_target_gbps.
+     **/
+    double disk_throughput_gbps;
+
+    /**
+     * Enable direct IO to bypass the OS cache. Helpful when the disk I/O outperforms the kernel cache.
+     * Notes:
+     * - Only supported on linux for now.
+     * - Only supports upload for now.
+     * - Check NOTES for O_DIRECT for additional info https://man7.org/linux/man-pages/man2/openat.2.html
+     * In summary, O_DIRECT is a potentially powerful tool that should be used with caution.
+     */
+    bool direct_io;
+};
+
 /**
  * Info about a single part, for you to review before the upload completes.
  */
@@ -438,6 +475,20 @@ struct aws_s3_client_config {
      */
     const struct aws_tls_connection_options *tls_connection_options;
 
+    /**
+     * Optional.
+     * If set, this controls how the client interact with file I/O.
+     * Read `aws_s3_file_io_options` for details.
+     *  Notes: Only applies to meta requests with `send_filepath` set.
+     *  TODO: adapt it to `recv_filepath`.
+     *
+     * eg:
+     * - When the file is too large to fit in the buffer, set `should_stream` to avoid buffering the whole parts in
+     *  memory.
+     * - When the disk I/O is faster than OS cache, set `direct_io` to bypass the OS cache.
+     */
+    struct aws_s3_file_io_options *fio_opts;
+
     /**
      * Required.
      * Configure the signing for the requests made from the client.
@@ -467,9 +518,7 @@ struct aws_s3_client_config {
      */
     uint64_t part_size;
 
-    /* If the part size needs to be adjusted for service limits, this is the maximum size it will be adjusted to. On 32
-     * bit machine, it will be forced to SIZE_MAX, which is around 4GiB. The server limit is 5GiB, but object size limit
-     * is 5TiB for now. We should be good enough for all the cases. */
+    /* If the part size needs to be adjusted for service limits, this is the maximum size it will be adjusted to. */
     uint64_t max_part_size;
 
     /**
@@ -737,7 +786,7 @@ struct aws_s3_meta_request_options {
      * If set, the received data will be written into this file.
      * the `body_callback` will NOT be invoked.
      * This gives a better performance when receiving data to write to a file.
-     * See `aws_s3_recv_file_option` for the configuration on the receive file.
+     * See `aws_s3_recv_file_options` for the configuration on the receive file.
      */
     struct aws_byte_cursor recv_filepath;
 
@@ -745,9 +794,9 @@ struct aws_s3_meta_request_options {
      * Optional.
      * Default to AWS_S3_RECV_FILE_CREATE_OR_REPLACE.
      * This only works with recv_filepath set.
-     * See `aws_s3_recv_file_option`.
+     * See `aws_s3_recv_file_options`.
      */
-    enum aws_s3_recv_file_option recv_file_option;
+    enum aws_s3_recv_file_options recv_file_option;
     /**
      * Optional.
      * The specified position to start writing at for the recv file when `recv_file_option` is set to
@@ -768,6 +817,24 @@ struct aws_s3_meta_request_options {
      */
     struct aws_byte_cursor send_filepath;
 
+    /**
+     * Optional.
+     * Overrides the client config if set.
+     * If set, this controls how the meta request interact with file I/O.
+     * Read `aws_s3_file_io_options` for details.
+     *  Notes: Only applies when `send_filepath` is set.
+     *  TODO: adapt it to `recv_filepath`.
+     *
+     * Note: if both client and meta request don't set this, for objects larger than 2TiB, this will be set to a default
+     * options with `should_stream` to be True and others follow the default to avoid memory issues.
+     *
+     * eg:
+     * - When the file is too large to fit in the buffer, set `should_stream` to avoid buffering the whole parts in
+     *  memory.
+     * - When the disk I/O is faster than OS cache, set `direct_io` to bypass the OS cache.
+     */
+    struct aws_s3_file_io_options *fio_opts;
+
     /**
      * Optional - EXPERIMENTAL/UNSTABLE
      * If set, the request body comes from this async stream.
@@ -1533,6 +1600,10 @@ void aws_s3_request_metrics_get_request_type(
 AWS_S3_API
 int aws_s3_request_metrics_get_error_code(const struct aws_s3_request_metrics *metrics);
 
+/* Get retry attempt from request metrics. */
+AWS_S3_API
+uint32_t aws_s3_request_metrics_get_retry_attempt(const struct aws_s3_request_metrics *metrics);
+
 AWS_EXTERN_C_END
 AWS_POP_SANE_WARNING_LEVEL
 
